from __future__ import nested_scopes
# -*- mode:Python; tab-width: 4 -*-
"""Extract or transform links in CSS documents.

This module follows the design of html_parser and uses a few of
its functions. The CSS parsing is limited to external links which
use the format 'url(xxx)', and it doesn't differentiate html files
from graphic philes, all will be processed as a graphic file (ie:
no intra-extension substitution).

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""

__author__ = "Grzegorz Adam Hankiewicz <gradha@efaber.net>"
__date__ = "May 2003"
__version__ = "$Revision: 1.5 $"
__credits__ = ""

import re
import urlparse
import html_parser

_css_regex = re.compile((r"url\((?P<url>.*?)\)"))

class Link_extractor:
    """Extract links from CSS code.
 
    Processes CSS code and stores all the external links, which
    you can get after the processing has finished.
    """
    def __init__(self, data, dummy = 0):
        """Create the object parsing CSS immediately.

        After the object has been initialised, the object processes
        data, which is presumed to be a string containing the whole
        CSS code of a style sheet.

        Once the object is returned you can extract its links or
        the processed CSS with the get_links and get_data methods.

        The dummy parameter there to mimic the interface of the
        same class in the html_parser module. It is not used.
        """
        self._data = []
        self._links = {}
        self.feed(data)
        self.close()

    def get_data(self):
        """Returns the processed CSS."""
        return self._data

    def get_links(self):
        """Returns all the found external links."""
        return self._links.keys()

    def feed(self, data):
        assert type(self._data) == type([])
        self._data.append(data)

    def close(self):
        self._data = "".join(self._data)
        for url in re.findall(_css_regex, self._data):
            self._links[url] = 1

class Link_transformer(Link_extractor):
    """Transforms all the links in a given CSS document.

    Inherits from Link_extractor, working pretty much the same. The
    only difference is that this time you pass the constructors a
    few more parameters and links will be modified.
    """
    def __init__(self, local_file, url, substitutions, dummy = 0):
        """Create the object parsing HTML immediately.

        After the object has been initialised, the object processes
        a CSS file.
  
        local_file is the path to the local file system where the
        CSS file is stored. url is the original URL of the very
        same file before it was saved to disk.
  
        substitutions is a dictionary. Its keys are the absolute
        URLs found in the opened file, and they should map to the
        absolute location of the local file, if any exists. Example:

        {"http://www.efaber.net/pixel.gif": "out/test/pixel.gif",
         "http://www.google.com/banner.png": "out/test/banner.png"}

        Don't worry about CSS code which uses relative links, that's
        why you give the url parameter in the constructor. The
        code first converts all relative links to absolute with
        the url parameter and *then* the link is searched in the
        substitution dictionery.  If a link is not found in the
        dictionary, it will be left as an external absolute link.

        Once the object is returned you can extract the modified
        processed code with the get_data method.
        """
        file = open(local_file, "rb")
        data = file.read()
        file.close()

        self._change = substitutions
        self._url = url
        self._local_file = local_file

        Link_extractor.__init__(self, data, dummy)
        
    def close(self):
        """Closes the stream and substitutes URLs.

        As soon as the stream has eneded, this method will build a
        regular expression containing all the substitutions and
        will apply it over all the found links.
        """
        Link_extractor.close(self)

        # create new dictionary with found links pointing to
        # relative or absolute URLs
        dict = {}
        for key in self._links.keys():
            url, fragment = html_parser.split_url(key)
            url = urlparse.urljoin(self._url, url)
            value = "%s%s" % (url, fragment)
            try:
                path_to = self._change[url]
                value = html_parser.relative_path(self._local_file,
                    path_to, fragment)
            except KeyError:
                pass
            dict[key] = value

        # the following code was extracted from
        # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/81330
        # the key reversing is to have the longer strings match
        # first in the OR regular expression.
        keys = dict.keys()
        keys.sort()
        keys.reverse()
        # create a regular expression from the dictionary keys
        regex = re.compile("(%s)" % "|".join(map(re.escape, keys)))

        # for each match, look-up corresponding value in dictionary
        self._data = regex.sub(lambda mo: dict[mo.string[mo.start():mo.end()]],
            self._data)
